/**
 * @since 1.0.0
 */
import { type CancelQueryCommandInput, type CancelQueryCommandOutput, type CreateScheduledQueryCommandInput, type CreateScheduledQueryCommandOutput, type DeleteScheduledQueryCommandInput, type DeleteScheduledQueryCommandOutput, type DescribeAccountSettingsCommandInput, type DescribeAccountSettingsCommandOutput, type DescribeEndpointsCommandInput, type DescribeEndpointsCommandOutput, type DescribeScheduledQueryCommandInput, type DescribeScheduledQueryCommandOutput, type ExecuteScheduledQueryCommandInput, type ExecuteScheduledQueryCommandOutput, type ListScheduledQueriesCommandInput, type ListScheduledQueriesCommandOutput, type ListTagsForResourceCommandInput, type ListTagsForResourceCommandOutput, type PrepareQueryCommandInput, type PrepareQueryCommandOutput, type QueryCommandInput, type QueryCommandOutput, type TagResourceCommandInput, type TagResourceCommandOutput, type TimestreamQueryClient, type TimestreamQueryClientConfig, type UntagResourceCommandInput, type UntagResourceCommandOutput, type UpdateAccountSettingsCommandInput, type UpdateAccountSettingsCommandOutput, type UpdateScheduledQueryCommandInput, type UpdateScheduledQueryCommandOutput } from "@aws-sdk/client-timestream-query";
import type { HttpHandlerOptions, ServiceLogger } from "@effect-aws/commons";
import type { Cause } from "effect";
import { Effect, Layer } from "effect";
import type { AccessDeniedError, ConflictError, InternalServerError, InvalidEndpointError, QueryExecutionError, ResourceNotFoundError, SdkError, ServiceQuotaExceededError, ThrottlingError, ValidationError } from "./Errors.js";
import * as Instance from "./TimestreamQueryClientInstance.js";
interface TimestreamQueryService$ {
    readonly _: unique symbol;
    /**
     * @see {@link CancelQueryCommand}
     */
    cancelQuery(args: CancelQueryCommandInput, options?: HttpHandlerOptions): Effect.Effect<CancelQueryCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | InternalServerError | InvalidEndpointError | ThrottlingError | ValidationError>;
    /**
     * @see {@link CreateScheduledQueryCommand}
     */
    createScheduledQuery(args: CreateScheduledQueryCommandInput, options?: HttpHandlerOptions): Effect.Effect<CreateScheduledQueryCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | ConflictError | InternalServerError | InvalidEndpointError | ServiceQuotaExceededError | ThrottlingError | ValidationError>;
    /**
     * @see {@link DeleteScheduledQueryCommand}
     */
    deleteScheduledQuery(args: DeleteScheduledQueryCommandInput, options?: HttpHandlerOptions): Effect.Effect<DeleteScheduledQueryCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | InternalServerError | InvalidEndpointError | ResourceNotFoundError | ThrottlingError | ValidationError>;
    /**
     * @see {@link DescribeAccountSettingsCommand}
     */
    describeAccountSettings(args: DescribeAccountSettingsCommandInput, options?: HttpHandlerOptions): Effect.Effect<DescribeAccountSettingsCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | InternalServerError | InvalidEndpointError | ThrottlingError>;
    /**
     * @see {@link DescribeEndpointsCommand}
     */
    describeEndpoints(args: DescribeEndpointsCommandInput, options?: HttpHandlerOptions): Effect.Effect<DescribeEndpointsCommandOutput, Cause.TimeoutException | SdkError | InternalServerError | ThrottlingError | ValidationError>;
    /**
     * @see {@link DescribeScheduledQueryCommand}
     */
    describeScheduledQuery(args: DescribeScheduledQueryCommandInput, options?: HttpHandlerOptions): Effect.Effect<DescribeScheduledQueryCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | InternalServerError | InvalidEndpointError | ResourceNotFoundError | ThrottlingError | ValidationError>;
    /**
     * @see {@link ExecuteScheduledQueryCommand}
     */
    executeScheduledQuery(args: ExecuteScheduledQueryCommandInput, options?: HttpHandlerOptions): Effect.Effect<ExecuteScheduledQueryCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | InternalServerError | InvalidEndpointError | ResourceNotFoundError | ThrottlingError | ValidationError>;
    /**
     * @see {@link ListScheduledQueriesCommand}
     */
    listScheduledQueries(args: ListScheduledQueriesCommandInput, options?: HttpHandlerOptions): Effect.Effect<ListScheduledQueriesCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | InternalServerError | InvalidEndpointError | ThrottlingError | ValidationError>;
    /**
     * @see {@link ListTagsForResourceCommand}
     */
    listTagsForResource(args: ListTagsForResourceCommandInput, options?: HttpHandlerOptions): Effect.Effect<ListTagsForResourceCommandOutput, Cause.TimeoutException | SdkError | InvalidEndpointError | ResourceNotFoundError | ThrottlingError | ValidationError>;
    /**
     * @see {@link PrepareQueryCommand}
     */
    prepareQuery(args: PrepareQueryCommandInput, options?: HttpHandlerOptions): Effect.Effect<PrepareQueryCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | InternalServerError | InvalidEndpointError | ThrottlingError | ValidationError>;
    /**
     * @see {@link QueryCommand}
     */
    query(args: QueryCommandInput, options?: HttpHandlerOptions): Effect.Effect<QueryCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | ConflictError | InternalServerError | InvalidEndpointError | QueryExecutionError | ThrottlingError | ValidationError>;
    /**
     * @see {@link TagResourceCommand}
     */
    tagResource(args: TagResourceCommandInput, options?: HttpHandlerOptions): Effect.Effect<TagResourceCommandOutput, Cause.TimeoutException | SdkError | InvalidEndpointError | ResourceNotFoundError | ServiceQuotaExceededError | ThrottlingError | ValidationError>;
    /**
     * @see {@link UntagResourceCommand}
     */
    untagResource(args: UntagResourceCommandInput, options?: HttpHandlerOptions): Effect.Effect<UntagResourceCommandOutput, Cause.TimeoutException | SdkError | InvalidEndpointError | ResourceNotFoundError | ThrottlingError | ValidationError>;
    /**
     * @see {@link UpdateAccountSettingsCommand}
     */
    updateAccountSettings(args: UpdateAccountSettingsCommandInput, options?: HttpHandlerOptions): Effect.Effect<UpdateAccountSettingsCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | InternalServerError | InvalidEndpointError | ThrottlingError | ValidationError>;
    /**
     * @see {@link UpdateScheduledQueryCommand}
     */
    updateScheduledQuery(args: UpdateScheduledQueryCommandInput, options?: HttpHandlerOptions): Effect.Effect<UpdateScheduledQueryCommandOutput, Cause.TimeoutException | SdkError | AccessDeniedError | InternalServerError | InvalidEndpointError | ResourceNotFoundError | ThrottlingError | ValidationError>;
}
/**
 * @since 1.0.0
 * @category constructors
 */
export declare const makeTimestreamQueryService: Effect.Effect<TimestreamQueryService$, never, Instance.TimestreamQueryClientInstance>;
declare const TimestreamQueryService_base: import("effect/Context").TagClass<TimestreamQueryService, "@effect-aws/client-timestream-query/TimestreamQueryService", TimestreamQueryService$> & Effect.Tag.Proxy<TimestreamQueryService, TimestreamQueryService$> & {
    use: <X>(body: (_: TimestreamQueryService$) => X) => [X] extends [Effect.Effect<infer A, infer E, infer R>] ? Effect.Effect<A, E, TimestreamQueryService | R> : [X] extends [PromiseLike<infer A_1>] ? Effect.Effect<A_1, Cause.UnknownException, TimestreamQueryService> : Effect.Effect<X, never, TimestreamQueryService>;
};
/**
 * @since 1.0.0
 * @category models
 */
export declare class TimestreamQueryService extends TimestreamQueryService_base {
    static readonly defaultLayer: Layer.Layer<TimestreamQueryService, never, never>;
    static readonly layer: (config: TimestreamQueryService.Config) => Layer.Layer<TimestreamQueryService, never, never>;
    static readonly baseLayer: (evaluate: (defaultConfig: TimestreamQueryClientConfig) => TimestreamQueryClient) => Layer.Layer<TimestreamQueryService, never, never>;
}
/**
 * @since 1.0.0
 */
export declare namespace TimestreamQueryService {
    /**
     * @since 1.0.0
     */
    interface Config extends Omit<TimestreamQueryClientConfig, "logger"> {
        readonly logger?: ServiceLogger.ServiceLoggerConstructorProps | true;
    }
}
export {};
//# sourceMappingURL=TimestreamQueryService.d.ts.map