/**
 * @since 1.0.0
 */
import { CancelQueryCommand, CreateScheduledQueryCommand, DeleteScheduledQueryCommand, DescribeAccountSettingsCommand, DescribeEndpointsCommand, DescribeScheduledQueryCommand, ExecuteScheduledQueryCommand, ListScheduledQueriesCommand, ListTagsForResourceCommand, PrepareQueryCommand, QueryCommand, TagResourceCommand, UntagResourceCommand, UpdateAccountSettingsCommand, UpdateScheduledQueryCommand, } from "@aws-sdk/client-timestream-query";
import { Service } from "@effect-aws/commons";
import { Effect, Layer } from "effect";
import { AllServiceErrors } from "./Errors.js";
import * as Instance from "./TimestreamQueryClientInstance.js";
import * as TimestreamQueryServiceConfig from "./TimestreamQueryServiceConfig.js";
const commands = {
    CancelQueryCommand,
    CreateScheduledQueryCommand,
    DeleteScheduledQueryCommand,
    DescribeAccountSettingsCommand,
    DescribeEndpointsCommand,
    DescribeScheduledQueryCommand,
    ExecuteScheduledQueryCommand,
    ListScheduledQueriesCommand,
    ListTagsForResourceCommand,
    PrepareQueryCommand,
    QueryCommand,
    TagResourceCommand,
    UntagResourceCommand,
    UpdateAccountSettingsCommand,
    UpdateScheduledQueryCommand,
};
/**
 * @since 1.0.0
 * @category constructors
 */
export const makeTimestreamQueryService = Effect.gen(function* () {
    const client = yield* Instance.TimestreamQueryClientInstance;
    return yield* Service.fromClientAndCommands(client, commands, {
        errorTags: AllServiceErrors,
        resolveClientConfig: TimestreamQueryServiceConfig.toTimestreamQueryClientConfig,
    });
});
/**
 * @since 1.0.0
 * @category models
 */
export class TimestreamQueryService extends Effect.Tag("@effect-aws/client-timestream-query/TimestreamQueryService")() {
    static defaultLayer = Layer.effect(this, makeTimestreamQueryService).pipe(Layer.provide(Instance.layer));
    static layer = (config) => Layer.effect(this, makeTimestreamQueryService).pipe(Layer.provide(Instance.layer), Layer.provide(TimestreamQueryServiceConfig.setTimestreamQueryServiceConfig(config)));
    static baseLayer = (evaluate) => Layer.effect(this, makeTimestreamQueryService).pipe(Layer.provide(Layer.effect(Instance.TimestreamQueryClientInstance, Effect.map(TimestreamQueryServiceConfig.toTimestreamQueryClientConfig, evaluate))));
}
//# sourceMappingURL=TimestreamQueryService.js.map