import { ServiceLogger } from "@effect-aws/commons";
import { Effect, FiberRef, Layer } from "effect";
import { dual } from "effect/Function";
import { globalValue } from "effect/GlobalValue";
/**
 * @since 1.0.0
 * @category timestream-query service config
 */
const currentTimestreamQueryServiceConfig = globalValue("@effect-aws/client-timestream-query/currentTimestreamQueryServiceConfig", () => FiberRef.unsafeMake({}));
/**
 * @since 1.0.0
 * @category timestream-query service config
 */
export const withTimestreamQueryServiceConfig = dual(2, (effect, config) => Effect.locally(effect, currentTimestreamQueryServiceConfig, config));
/**
 * @since 1.0.0
 * @category timestream-query service config
 */
export const setTimestreamQueryServiceConfig = (config) => Layer.locallyScoped(currentTimestreamQueryServiceConfig, config);
/**
 * @since 1.0.0
 * @category adapters
 */
export const toTimestreamQueryClientConfig = Effect.gen(function* () {
    const { logger: serviceLogger, ...config } = yield* FiberRef.get(currentTimestreamQueryServiceConfig);
    const logger = serviceLogger === true
        ? yield* ServiceLogger.toClientLogger(ServiceLogger.defaultServiceLogger)
        : (serviceLogger ? yield* ServiceLogger.toClientLogger(ServiceLogger.make(serviceLogger)) : undefined);
    return { logger, ...config };
});
//# sourceMappingURL=TimestreamQueryServiceConfig.js.map